@@ -77,11 +77,9 @@ void                     clientGrabMouseButton                  (Client *);
 void                     clientUngrabMouseButton                (Client *);
 void                     clientGrabMouseButtonForAll            (ScreenInfo *);
 void                     clientUngrabMouseButtonForAll          (ScreenInfo *);
-void                     clientPassGrabMouseButton              (Client *);
-Client                  *clientGetLastUngrab                    (void);
-void                     clientClearLastUngrab                  (void);
 void                     clientClearDelayedFocus                (void);
-void                     clientAddDelayedFocus                  (Client *);
+void                     clientAddDelayedFocus                  (Client *,
+                                                                 guint32);
 Client                  *clientGetDelayedFocus                  (void);
 
 #endif /* INC_FOCUS_H */
