@@ -58,6 +58,12 @@ unsigned int ScrollLockMask;
 unsigned int SuperMask;
 unsigned int HyperMask;
 
+#define BUTTON_GRAB_MASK \
+    ButtonPressMask | \
+    ButtonReleaseMask | \
+    PointerMotionMask | \
+    PointerMotionHintMask
+
 static KeyCode
 getKeycode (Display *dpy, const char *str)
 {
@@ -141,31 +147,31 @@ grabKey (Display * dpy, MyKey * key, Window w)
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | ScrollLockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | NumLockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | LockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | ScrollLockMask | NumLockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | ScrollLockMask | LockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | LockMask | NumLockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
         status |=
             XGrabKey (dpy, key->keycode,
                                     key->modifier | ScrollLockMask | LockMask | NumLockMask, w,
-                                    TRUE, GrabModeAsync, GrabModeAsync);
+                                    TRUE, GrabModeAsync, GrabModeSync);
     }
 
     return (status == GrabSuccess);
@@ -191,7 +197,7 @@ grabButton (Display * dpy, int button, int modifier, Window w)
     {
         status |=
             XGrabButton (dpy, button, AnyModifier, w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
     }
     else
@@ -200,42 +206,42 @@ grabButton (Display * dpy, int button, int modifier, Window w)
         status |=
             XGrabButton (dpy, button, modifier,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | ScrollLockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | NumLockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | LockMask, w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | ScrollLockMask | NumLockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | ScrollLockMask | LockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button, modifier | LockMask | NumLockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
         status |=
             XGrabButton (dpy, button,
                                 modifier | ScrollLockMask | LockMask | NumLockMask,
                                 w, FALSE,
-                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                BUTTON_GRAB_MASK, GrabModeSync, GrabModeAsync,
                                 None, None);
     }
 
